Package: imcRtools
Version: 1.19.1
Title: Methods for imaging mass cytometry data analysis
Description: 
    This R package supports the handling and analysis of imaging mass cytometry 
    and other highly multiplexed imaging data. The main functionality includes 
    reading in single-cell data after image segmentation and measurement, data 
    formatting to perform channel spillover correction and a number of spatial 
    analysis approaches. First, cell-cell interactions are detected via spatial 
    graph construction; these graphs can be visualized with cells representing 
    nodes and interactions representing edges. Furthermore, per cell, its direct 
    neighbours are summarized to allow spatial clustering. Per image/grouping 
    level, interactions between types of cells are counted, averaged and 
    compared against random permutations. In that way, types of cells that 
    interact more (attraction) or less (avoidance) frequently than expected by 
    chance are detected. 
Authors@R: 
    c(person("Nils", "Eling", role = "aut"),
      person("Tobias", "Hoch", role = "ctb"),
      person("Vito", "Zanotelli", role = "ctb"),
      person("Jana", "Fischer", role = "ctb"),
      person("Daniel", "Schulz", role = c("ctb", "cre"),email = "daniel.schulz@uzh.ch",comment = c(ORCID = "0000-0002-0913-1678")),
      person("Lasse","Meyer", role = "ctb"),
      person("Lutz","Marlene", role = "ctb"),
      person("Schiller","Chiara", role = "ctb"),
      person("Ibañez", "Victor", role = "ctb"))
License: GPL-3
Depends: R (>= 4.1), SpatialExperiment
Imports: S4Vectors, stats, utils, SummarizedExperiment, methods,
        pheatmap, scrapper, stringr, readr, EBImage, cytomapper, abind,
        BiocParallel, viridis, dplyr, magrittr, DT, igraph,
        SingleCellExperiment, vroom, BiocNeighbors, RTriangle, ggraph,
        tidygraph, ggplot2, data.table, sf, concaveman, tidyselect,
        distances, MatrixGenerics, rlang, grDevices
Suggests: CATALYST, grid, tidyr, BiocStyle, knitr, rmarkdown, markdown,
        testthat
biocViews: ImmunoOncology, SingleCell, Spatial, DataImport, Clustering
VignetteBuilder: knitr
URL: https://github.com/BodenmillerGroup/imcRtools
BugReports: https://github.com/BodenmillerGroup/imcRtools/issues
RoxygenNote: 7.3.3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/imcRtools
git_branch: devel
git_last_commit: 6b31abd
git_last_commit_date: 2026-05-09
Repository: Bioconductor 3.24
Date/Publication: 2026-05-10
NeedsCompilation: no
Packaged: 2026-05-10 22:37:27 UTC; biocbuild
Author: Nils Eling [aut],
  Tobias Hoch [ctb],
  Vito Zanotelli [ctb],
  Jana Fischer [ctb],
  Daniel Schulz [ctb, cre] (ORCID:
    <https://orcid.org/0000-0002-0913-1678>),
  Lasse Meyer [ctb],
  Lutz Marlene [ctb],
  Schiller Chiara [ctb],
  Ibañez Victor [ctb]
Maintainer: Daniel Schulz <daniel.schulz@uzh.ch>
